@@ -26,26 +26,26 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 import os
-import re
 from wtforms import Form
 from wtforms.fields import (SelectField, TextField, BooleanField, IntegerField,
                             PasswordField)
 from wtforms.widgets import PasswordInput
 from wtforms.validators import NumberRange, URL, EqualTo, Regexp, Optional
-from yubiadmin.util.app import App, render
+from yubiadmin.util.app import App, CollectionApp
 from yubiadmin.util.config import (python_handler, python_list_handler,
                                    FileConfig)
 from yubiadmin.util.form import ConfigForm, FileForm, ListField
 try:
     from yubiauth import YubiAuth
-    from yubiauth.core.model import User
 except:
     YubiAuth = None
 
+
 __all__ = [
     'app'
 ]
 
+
 AUTH_CONFIG_FILE = '/etc/yubico/auth/yubiauth.conf'
 YKVAL_SERVERS = [
     'https://api.yubico.com/wsapi/2.0/verify',
@@ -178,12 +178,16 @@ class YubiAuthApp(App):
 
     name = 'auth'
     sections = ['general', 'database', 'validation', 'advanced']
-    disabled = not os.path.isfile(AUTH_CONFIG_FILE)
 
-    def __init__(self):
-        if YubiAuth:
-            self._users = YubiAuthUsers()
-            self.sections.insert(3, 'users')
+    @property
+    def disabled(self):
+        return not os.path.isfile(AUTH_CONFIG_FILE)
+
+    @property
+    def sections(self):
+        if not YubiAuth:
+            return ['general', 'database', 'validation', 'advanced']
+        return ['general', 'database', 'validation', 'users', 'advanced']
 
     def general(self, request):
         """
@@ -208,14 +212,15 @@ class YubiAuthApp(App):
         Advanced
         """
         return self.render_forms(request, [
-            FileForm(AUTH_CONFIG_FILE, 'Configuration')
-        ])
+            FileForm(AUTH_CONFIG_FILE, 'Configuration', lang='python')
+        ], script='editor')
 
     def users(self, request):
         """
         Manage Users
         """
-        return self._users(request) if YubiAuth else ""
+        with YubiAuthUsers() as users:
+            return users(request)
 
     # Pulls the tab to the right:
     advanced.advanced = True
@@ -234,9 +239,6 @@ class CreateUserForm(Form):
         super(CreateUserForm, self).__init__(**kwargs)
         self.auth = auth
 
-    def load(self):
-        pass
-
     def save(self):
         self.auth.create_user(self.username.data, self.password.data)
         self.auth.commit()
@@ -291,70 +293,53 @@ class AssignYubiKeyForm(Form):
             self.auth.commit()
 
 
-class YubiAuthUsers(App):
-    user_range = re.compile('(\d+)-(\d+)')
+class YubiAuthUsers(CollectionApp):
+    base_url = '/auth/users'
+    item_name = 'Users'
+    caption = 'YubiAuth Users'
+    columns = ['Username', 'YubiKeys']
+    template = 'auth/list'
 
     def __init__(self):
+        from yubiauth.core.model import User as _user
+        self.User = _user
         self.auth = YubiAuth()
 
-    def __call__(self, request):
-        sub_cmd = request.path_info_pop()
-        if sub_cmd == 'create':
-            return self.create(request)
-        elif sub_cmd == 'delete':
-            return self.delete(request)
-        elif sub_cmd == 'delete_confirm':
-            return self.delete_confirm(request)
-        elif sub_cmd == 'user':
-            return self.show_user(request)
-        else:
-            match = self.user_range.match(sub_cmd) if sub_cmd else None
-            if match:
-                offset = int(match.group(1)) - 1
-                limit = int(match.group(2)) - offset
-            else:
-                offset = 0
-                limit = 10
-        return self.list_users(offset, limit)
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        del self.auth
+
+    def _size(self):
+        return self.auth.session.query(self.User).count()
+
+    def _get(self, offset=0, limit=None):
+        users = self.auth.session.query(self.User).order_by(self.User.name) \
+            .offset(offset).limit(limit)
+
+        return map(lambda user: {
+            'id': user.id,
+            'label': user.name,
+            'Username': '<a href="/auth/users/show/%d">%s</a>' % (user.id,
+                                                                  user.name),
+            'YubiKeys': ', '.join(user.yubikeys.keys())
+        }, users)
+
+    def _select(self, ids):
+        return self.auth.session.query(self.User.name, self.User.id) \
+            .filter(self.User.id.in_(map(int, ids))).all()
+
+    def _delete(self, ids):
+        self.auth.session.query(self.User) \
+            .filter(self.User.id.in_(map(int, ids))).delete('fetch')
+        self.auth.commit()
 
     def create(self, request):
         return self.render_forms(request, [CreateUserForm(self.auth)],
                                  success_msg='User created!')
 
-    def delete(self, request):
-        ids = [int(x[5:]) for x in request.params if request.params[x] == 'on']
-        users = self.auth.session.query(User.name, User.id) \
-            .filter(User.id.in_(ids)).all()
-        return render('auth/delete', users=users)
-
-    def delete_confirm(self, request):
-        ids = [int(x) for x in request.params['delete'].split(',')]
-        self.auth.session.query(User).filter(User.id.in_(ids)).delete('fetch')
-        self.auth.commit()
-        return self.redirect('/auth/users')
-
-    def list_users(self, offset, limit):
-        users = self.auth.session.query(User).order_by(User.name) \
-            .offset(offset).limit(limit)
-        num_users = self.auth.session.query(User).count()
-        shown = (min(offset + 1, num_users), min(offset + limit, num_users))
-        if offset > 0:
-            st = max(0, offset - limit)
-            ed = st + limit
-            prev = '/auth/users/%d-%d' % (st + 1, ed)
-        else:
-            prev = None
-        if num_users > shown[1]:
-            next = '/auth/users/%d-%d' % (offset + limit + 1, shown[1] + limit)
-        else:
-            next = None
-
-        return render(
-            'auth/list', script='auth', users=users, offset=offset,
-            limit=limit, num_users=num_users, shown='%d-%d' % shown, prev=prev,
-            next=next)
-
-    def show_user(self, request):
+    def show(self, request):
         id = int(request.path_info_pop())
         user = self.auth.get_user(id)
         if 'unassign' in request.params:
